"""Logging fuctions."""
import sys
import logging

LOGFORMAT_VERBOSE = "%(levelname)-7s: %(message)-80s [%(filename)s L%(lineno)d]"

LOGFORMAT = "%(levelname)-7s: %(message)-80s"


def init_logger(level=logging.DEBUG, verbose=False):
    """Initialise the logger."""
    logger = logging.getLogger("")
    handler = logging.StreamHandler(sys.stdout)
    if verbose:
        formatter = logging.Formatter(LOGFORMAT_VERBOSE)
    else:
        formatter = logging.Formatter(LOGFORMAT)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(level)



"""SDC2 scoring service admin API functions."""
import os
import sys
import json
import http.client
from http import HTTPStatus
import urllib.parse
import logging
from keycloak import KeycloakOpenID

LOG = logging.getLogger(__name__)

SDC_SCORE_API_HOST = os.getenv("SDC_SCORE_API_HOST", "130.246.212.180")
SDC_SCORE_API_PORT = os.getenv("SDC_SCORE_API_PORT", "3000")
SDC_SCORE_AUTH_HOST = os.getenv("SDC_SCORE_AUTH_HOST", "130.246.212.32")
SDC_SCORE_AUTH_PORT = os.getenv("SDC_SCORE_AUTH_PORT", "8080")
SDC_SCORE_AUTH_CLIENT_ID = "sdc"
SDC_SCORE_AUTH_REALM_NAME = "sdc"


def get_admin_token():
    """Get keyclock admin token."""
    user = os.getenv("SDC2_SCORER_ADMIN_USER", None)
    passwd = os.getenv("SDC2_SCORER_ADMIN_PASSWORD", None)
    if not user or not passwd:
        LOG.error(
            "In order to use this app, please set the env varaibles: SDC2_SCORER_ADMIN_USER and SDC2_SCORER_ADMIN_PASSWORD"
        )
        sys.exit(1)
    auth_url = f"http://{SDC_SCORE_AUTH_HOST}:{SDC_SCORE_AUTH_PORT}/auth/"
    LOG.debug(f"auth url = {auth_url}")
    LOG.debug(f"getting auth token... {user} - {passwd}")
    keycloak_openid = KeycloakOpenID(
        auth_url,
        client_id=SDC_SCORE_AUTH_CLIENT_ID,
        realm_name=SDC_SCORE_AUTH_REALM_NAME,
    )
    token = keycloak_openid.token(username=user, password=passwd)
    return token


def get_config(endpoint):
    """Return json configuration data at specified endpoint."""
    token = get_admin_token()
    conn = http.client.HTTPConnection(SDC_SCORE_API_HOST, SDC_SCORE_API_PORT)
    _headers = {"Authorization": "Bearer " + token["access_token"]}
    conn.request("GET", endpoint, headers=_headers)
    resp = conn.getresponse()
    LOG.debug("%s %s", resp.status, resp.reason)
    data = resp.read().decode("utf-8")
    conn.close()
    return json.loads(data)


def get_users_group(user_group_uuid):
    """Return group for specified user."""
    groups = get_config("/groups")
    try:
        group = next(
            group_data for group_data in groups if group_data[0] == user_group_uuid
        )
        return group[1]
    except StopIteration:
        return None


def get_group_uuid(group_name):
    """Return the group uuid for a specified group name."""
    groups = get_config("/groups")
    try:
        uuid = next(
            group_data[0] for group_data in groups if group_data[1] == group_name
        )
        return uuid
    except StopIteration:
        return None


def get_group_name(group_uuid):
    """Return the group name for a specified group uuid."""
    groups = get_config("/groups")
    try:
        name = next(group[1] for group in groups if group[0] == group_uuid)
        return name
    except StopIteration:
        return None


def get_user_uuid(username: str):
    """Return the user uuid for a specified username."""
    users = get_config("/users")
    try:
        uuid = next(user[0] for user in users if user[3] == username)
        return uuid
    except StopIteration:
        return None


def get_users_in_group(group_uuid: str):
    """Return the user uuid for a specified username."""
    all_users = get_config("/users")
    group_users = []
    for user in all_users:
        if user[5] == group_uuid:
            group_users.append(user)
    return group_users


# =============================================================================
# GROUP methods
# =============================================================================


def user_list():
    """Display list of users."""
    LOG.info("Users: <Last Name, First Name | username | email | uuid | group name>")
    for user in get_config("/users"):
        # LOG.info(f"   {user}")
        group = get_users_group(user[5])
        LOG.info(
            f"   {user[2]}, {user[1]} | {user[3]} | {user[4]} | {user[0]} | {group}"
        )


def user_add(
    first_name: str,
    last_name: str,
    username: str,
    password: str,
    email: str,
    group_name: str = None,
):
    """Add a new user.

    Args:
        first_name (str):
        last_name (str):
        username (str):
        password (str):
        email (str):
        group_name (str):

    TODO:
        - Dont allow adding two users with the same name!
    """
    LOG.info(
        "Adding user: %s %s (%s) | password: '%s'",
        first_name,
        last_name,
        username,
        password,
    )
    group_name = " ".join(group_name)
    group_uuid = get_group_uuid(group_name)
    if not group_uuid:
        LOG.error("Unable to add user, group with name '%s' not found!", group_name)
        return

    token = get_admin_token()
    conn = http.client.HTTPConnection(SDC_SCORE_API_HOST, SDC_SCORE_API_PORT)
    headers = {
        "Authorization": "Bearer " + token["access_token"],
        "Content-Type": "multipart/form-data",
        "Accept": "application/json",
    }
    params = urllib.parse.urlencode(
        {
            "firstName": first_name,
            "lastName": last_name,
            "username": username,
            "password": password,
            "email": email,
            "groupId": group_uuid,
        }
    )
    conn.request("POST", "/users" + "?" + params, headers=headers)
    resp = conn.getresponse()
    data = json.loads(resp.read().decode("utf-8"))
    conn.close()
    if resp.status == HTTPStatus.OK:
        LOG.info(
            "Sucessfully added user '%s %s (%s)', uuid = %s",
            first_name,
            last_name,
            username,
            data,
        )
    else:
        LOG.error("Failed to add user. %s : %s (%s)", resp.status, resp.reason, data)


def user_delete(username: str):
    """Delete a user.

    Args:
        username(str): Username

    """
    uuid = get_user_uuid(username)
    if not uuid:
        LOG.error("User id for username '%s' not found!", username)
        return

    LOG.info("Deleteing User: '%s' (%s)", username, uuid)
    token = get_admin_token()
    conn = http.client.HTTPConnection(SDC_SCORE_API_HOST, SDC_SCORE_API_PORT)
    headers = {
        "Authorization": "Bearer " + token["access_token"],
        "Content-Type": "multipart/form-data",
        "Accept": "application/json",
    }
    params = urllib.parse.urlencode({"userId": uuid})
    conn.request("DELETE", "/users" + "?" + params, headers=headers)
    resp = conn.getresponse()
    data = json.loads(resp.read().decode("utf-8"))
    conn.close()
    if resp.status == HTTPStatus.OK:
        LOG.info("Sucessfully deleted user '%s', uuid = %s", username, data)
    else:
        LOG.error(
            "Failed to delete user. '%s' : %s (%s)", resp.status, resp.reason, data
        )


# =============================================================================
# GROUP methods
# =============================================================================


def group_list(show_users=False):
    """Display list of groups."""
    groups = get_config("/groups")
    LOG.info("Groups%s:", " (and users in each group)" if show_users else "")
    for group in groups:
        LOG.info(f"  {group[1]:40s} (group id: {group[0]})")
        if show_users:
            users = get_users_in_group(group_uuid=group[0])
            for user in users:
                LOG.info(f"    {user[1]} {user[2]:40s} (user id: {user[0]})")


def group_add(name: str):
    """Add a new group.

    Args:
        name (str): Name of the group to add.
    """
    name = " ".join(name)
    LOG.info("Adding group: '%s'", name)
    if get_group_uuid(name):
        LOG.error("Ignoring request, group already exists!")
    token = get_admin_token()
    conn = http.client.HTTPConnection(SDC_SCORE_API_HOST, SDC_SCORE_API_PORT)
    headers = {
        "Authorization": "Bearer " + token["access_token"],
        "Content-Type": "multipart/form-data",
        "Accept": "application/json",
    }
    params = urllib.parse.urlencode({"name": name})
    conn.request("POST", "/groups" + "?" + params, headers=headers)
    resp = conn.getresponse()
    data = json.loads(resp.read().decode("utf-8"))
    conn.close()
    if resp.status == HTTPStatus.OK:
        LOG.info("Sucessfully added group '%s', uuid = %s", name, data)
    else:
        LOG.error("Failed to add group. %s : %s (%s)", resp.status, resp.reason, data)


def group_delete(name: str):
    """Remove a group.

    Args:
        name(str): Group name

    """
    name = " ".join(name)
    uuid = get_group_uuid(name)
    LOG.info("Deleteing group: '%s' (%s)", name, uuid)
    token = get_admin_token()
    conn = http.client.HTTPConnection(SDC_SCORE_API_HOST, SDC_SCORE_API_PORT)
    headers = {
        "Authorization": "Bearer " + token["access_token"],
        "Content-Type": "multipart/form-data",
        "Accept": "application/json",
    }
    params = urllib.parse.urlencode({"groupId": uuid})
    conn.request("DELETE", "/groups" + "?" + params, headers=headers)
    resp = conn.getresponse()
    data = json.loads(resp.read().decode("utf-8"))
    conn.close()
    if resp.status == HTTPStatus.OK:
        LOG.info("Sucessfully deleted group '%s', uuid = %s", name, data)
    else:
        LOG.error(
            "Failed to delete group. '%s' : %s (%s)", resp.status, resp.reason, data
        )



"""Packaging script for sdc2 scoring utilities.

https://packaging.python.org/tutorials/packaging-projects/
"""
from setuptools import setup, find_packages

with open("README.md", "r") as file:
    README = file.read()

setup(
    author="SKA Organisation",
    name="ska-sdc2-scoring-utils",
    version="0.1.1-rc1",
    description="Utility scripts for interacting with SKA SDC2 scoring service.",
    url="https://gitlab.com/ska-telescope/sdc/sdc2-scoring-utils",
    license="License :: OSI Approved :: BSD License",
    long_description=README,
    long_description_content_type="text/markdown",
    scripts=["scripts/sdc2-score", "scripts/sdc2-score-admin"],
    package_dir={"": "src"},
    packages=find_packages("src"),
    install_requires=["python-keycloak", "requests"],
    python_rquires=">=3.6",
    clasifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Astronomy",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
    ],
)



"""SDC2 scoring service API functions."""
import os
import json
import http.client
from http import HTTPStatus
import urllib.parse
import logging
from keycloak import KeycloakOpenID
from requests.models import encode_multipart_formdata
import requests


LOG = logging.getLogger(__name__)

SDC_SCORE_API_HOST = os.getenv("SDC_SCORE_API_HOST", "130.246.212.180")
SDC_SCORE_API_PORT = os.getenv("SDC_SCORE_API_PORT", "3000")
SDC_SCORE_AUTH_HOST = os.getenv("SDC_SCORE_AUTH_HOST", "130.246.212.32")
SDC_SCORE_AUTH_PORT = os.getenv("SDC_SCORE_AUTH_PORT", "8080")
SDC_SCORE_AUTH_CLIENT_ID = "sdc"
SDC_SCORE_AUTH_REALM_NAME = "sdc"


def get_token(username: str, password: str):
    """Get keyclock token."""
    auth_url = f"http://{SDC_SCORE_AUTH_HOST}:{SDC_SCORE_AUTH_PORT}/auth/"
    LOG.debug(f"Auth url = {auth_url}")
    LOG.debug(f"Getting auth token... {username} - {password}")
    keycloak_openid = KeycloakOpenID(
        auth_url,
        client_id=SDC_SCORE_AUTH_CLIENT_ID,
        realm_name=SDC_SCORE_AUTH_REALM_NAME,
    )
    token = keycloak_openid.token(username=username, password=password)
    return token


def get_config(endpoint: str):
    """Return json configuration data at specified endpoint."""
    conn = http.client.HTTPConnection(SDC_SCORE_API_HOST, SDC_SCORE_API_PORT)
    conn.request("GET", endpoint)
    resp = conn.getresponse()
    LOG.debug("%s %s", resp.status, resp.reason)
    data = resp.read().decode("utf-8")
    conn.close()
    return json.loads(data)


def get_user_uuid(username: str):
    """Return the user uuid for a specified username."""
    users = get_config("/users")
    try:
        uuid = next(user[0] for user in users if user[3] == username)
        return uuid
    except StopIteration:
        return None


def display_leaderboard():
    """Display the leaderboard."""
    leaderboard = get_config("/sdc2/leaderboard")
    LOG.info("Leaderboard:")
    for score in leaderboard:
        LOG.info(score)


def create_submission(username, password, catalogue_file):
    """Create submision."""
    LOG.info("Authenticating as user: %s", username)
    auth_token = get_token(username, password)
    LOG.info("Creating submission from file: %s", catalogue_file)
    headers = {"Authorization": "Bearer " + auth_token["access_token"]}
    files = {
        "submissionFile": (
            catalogue_file,
            open(catalogue_file, "rb"),
            "text/plain",
        )
    }
    params = urllib.parse.urlencode(
        {
            "subSkipRows": 0,
            "truthSkipRows": 0,
        }
    )
    url = f"http://{SDC_SCORE_API_HOST}:{SDC_SCORE_API_PORT}"
    url += "/sdc2/v1/submission"
    url += "?" + params
    resp = requests.post(url, files=files, headers=headers, timeout=10)
    if resp.status_code == HTTPStatus.OK.value:
        LOG.info("Submission sucessful!")
        LOG.info("Submission id: %s", resp.text)
        LOG.info("(Please take a note of this as it is needed to check the status!)")
    else:
        LOG.error("Failed to create submission. %s : %s", resp.status_code, resp.text)


def display_submission_status(submisson_uuid: str):
    """Get the submission status."""
    LOG.info("Getting submission status: %s", submisson_uuid)
    conn = http.client.HTTPConnection(SDC_SCORE_API_HOST, SDC_SCORE_API_PORT)
    params = urllib.parse.urlencode({"submissionId": submisson_uuid})
    conn.request("GET", "/sdc2/v1/submission" + "?" + params)
    resp = conn.getresponse()
    if resp.status != HTTPStatus.OK:
        LOG.error("Failed to get status. %s : %s", resp.status, resp.reason)
    data = json.loads(resp.read().decode("utf-8"))
    conn.close()
    LOG.info("Status = %s (%s)", data[5], data[4])
